# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):
    initial = True
    
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Measure',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('value', models.IntegerField()),
                ('measure_type', models.IntegerField(choices=[(1, b'Weight (kg)'), (4, b'Height (meter)'), (5, b'Fat Free Mass (kg)'), (6, b'Fat Ratio (%)'), (8, b'Fat Mass Weight (kg)'), (9, b'Diastolic Blood Pressure (mmHg)'), (10, b'Systolic Blood Pressure (mmHg)'), (11, b'Heart Pulse (bpm)'), (54, b'SP02(%)')])),
                ('unit', models.IntegerField()),
            ],
            options={
                'db_table': 'withingsapp_measure',
            }
        ),
        migrations.CreateModel(
            name='MeasureGroup',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('grpid', models.IntegerField()),
                ('attrib', models.IntegerField(choices=[(0, b'Captured by a device, not ambiguous'), (1, b'Captured by a device, may belong to other user'), (2, b'Manually entered by user'), (4, b'Manually entered, may not be accurate')])),
                ('date', models.DateTimeField()),
                ('updatetime', models.DateTimeField()),
                ('category', models.IntegerField(choices=[(1, b'Real measurements'), (2, b'User objectives')])),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'withingsapp_measuregroup',
            }
        ),
        migrations.CreateModel(
            name='WithingsUser',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('withings_user_id', models.IntegerField()),
                ('access_token', models.TextField()),
                ('access_token_secret', models.TextField()),
                ('last_update', models.DateTimeField(null=True, blank=True)),
                ('user', models.OneToOneField(to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'withingsapp_withingsuser',
            }
        ),
        migrations.AddField(
            model_name='measure',
            name='group',
            field=models.ForeignKey(to='MeasureGroup'),
        ),
        migrations.AlterUniqueTogether(
            name='measuregroup',
            unique_together=set([('user', 'grpid')]),
        ),
    ]



from django.conf.urls import url

from . import views


urlpatterns = [
    # OAuth authentication
    url(r'^login/$', views.login, name='nokia-login'),
    url(r'^complete/$', views.complete, name='nokia-complete'),
    url(r'^error/$', views.error, name='nokia-error'),
    url(r'^logout/$', views.logout, name='nokia-logout'),

    # Subscriber callback for notifications
    url(r'^notification/(?P<appli>[14])/$', views.notification,
        name='nokia-notification')
]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2018-09-11 15:29
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('nokiaapp', '0003_rename_nokiaapp'),
    ]

    operations = [
        migrations.AddField(
            model_name='nokiauser',
            name='refresh_token',
            field=models.TextField(default='', help_text=b'OAuth2 refresh token'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='nokiauser',
            name='token_expiry',
            field=models.IntegerField(default=0, help_text=b'Token expiration timestamp'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='nokiauser',
            name='token_type',
            field=models.CharField(default='', help_text=b'Type of OAuth2 token', max_length=32),
            preserve_default=False,
        ),
    ]



# Your Nokia access credentials, which must be requested from Nokia.
# You must provide these in your project's settings.
NOKIA_CLIENT_ID = None
NOKIA_CONSUMER_SECRET = None

# Where to redirect to after Nokia authentication is successfully completed.
NOKIA_LOGIN_REDIRECT = '/'

# Where to redirect to after Nokia authentication credentials have been
# removed.
NOKIA_LOGOUT_REDIRECT = '/'

# By default, don't subscribe to user data. Set this to true to subscribe.
NOKIA_SUBSCRIBE = False

# The template to use when an unavoidable error occurs during Nokia
# integration.
NOKIA_ERROR_TEMPLATE = 'nokia/error.html'

# The default message used by the nokia_integration_warning decorator to
# inform the user about Nokia integration. If a callable is given, it is
# called with the request as the only parameter to get the final value for the
# message.
NOKIA_DECORATOR_MESSAGE = 'This page requires Nokia integration.'



import datetime

from django.contrib import messages
from django.contrib.auth.models import AnonymousUser
from django.core.urlresolvers import reverse
from django.http import HttpRequest
from django.utils import timezone
from freezegun import freeze_time
from nokia import NokiaApi, NokiaAuth, NokiaCredentials

from nokiaapp import utils
from nokiaapp.decorators import nokia_integration_warning
from nokiaapp.models import NokiaUser, MeasureGroup, Measure

from .base import NokiaTestBase

try:
    from unittest import mock
except ImportError:  # Python 2.x fallback
    import mock


class TestIntegrationUtility(NokiaTestBase):

    def test_is_integrated(self):
        """Users with stored OAuth information are integrated."""
        self.assertTrue(utils.is_integrated(self.user))

    def test_is_not_integrated(self):
        """User is not integrated if we have no OAuth data for them."""
        NokiaUser.objects.all().delete()
        self.assertFalse(utils.is_integrated(self.user))

    def test_unauthenticated(self):
        """User is not integrated if they aren't logged in."""
        user = AnonymousUser()
        self.assertFalse(utils.is_integrated(user))


class TestIntegrationDecorator(NokiaTestBase):

    def setUp(self):
        super(TestIntegrationDecorator, self).setUp()
        self.fake_request = HttpRequest()
        self.fake_request.user = self.user
        self.fake_view = lambda request: "hello"
        self.messages = []

    def _mock_decorator(self, msg=None):
        def mock_error(request, message, *args, **kwargs):
            self.messages.append(message)

        with mock.patch.object(messages, 'error', mock_error):
            return nokia_integration_warning(msg=msg)(self.fake_view)(
                self.fake_request)

    def test_unauthenticated(self):
        """Message should be added if user is not logged in."""
        self.fake_request.user = AnonymousUser()
        results = self._mock_decorator()

        self.assertEqual(results, "hello")
        self.assertEqual(len(self.messages), 1)
        self.assertEqual(self.messages[0],
                         utils.get_setting('NOKIA_DECORATOR_MESSAGE'))

    def test_is_integrated(self):
        """Decorator should have no effect if user is integrated."""
        results = self._mock_decorator()

        self.assertEqual(results, "hello")
        self.assertEqual(len(self.messages), 0)

    def test_is_not_integrated(self):
        """Message should be added if user is not integrated."""
        NokiaUser.objects.all().delete()
        results = self._mock_decorator()

        self.assertEqual(results, "hello")
        self.assertEqual(len(self.messages), 1)
        self.assertEqual(self.messages[0],
                         utils.get_setting('NOKIA_DECORATOR_MESSAGE'))

    def test_custom_msg(self):
        """Decorator should support a custom message string."""
        NokiaUser.objects.all().delete()
        msg = "customized"
        results = self._mock_decorator(msg)

        self.assertEqual(results, "hello")
        self.assertEqual(len(self.messages), 1)
        self.assertEqual(self.messages[0], "customized")

    def test_custom_msg_func(self):
        """Decorator should support a custom message function."""
        NokiaUser.objects.all().delete()

        def msg(request):
            return "message to {0}".format(request.user)
        results = self._mock_decorator(msg)

        self.assertEqual(results, "hello")
        self.assertEqual(len(self.messages), 1)
        self.assertEqual(self.messages[0], msg(self.fake_request))


class TestLoginView(NokiaTestBase):
    url_name = 'nokia-login'

    def setUp(self):
        super(TestLoginView, self).setUp()
        NokiaAuth.get_authorize_url = mock.MagicMock(return_value='/test')

    def test_get(self):
        """
        Login view should redirect to an authorization URL and create a user
        """
        response = self._get()
        self.assertRedirectsNoFollow(response, '/test')
        self.assertEqual(NokiaUser.objects.count(), 1)

    def test_unauthenticated(self):
        """User must be logged in to access Login view."""
        self.client.logout()
        response = self._get()
        self.assertEqual(response.status_code, 302)
        self.assertEqual(NokiaUser.objects.count(), 1)

    def test_unintegrated(self):
        """Nokia credentials not required to access Login view."""
        self.nokia_user.delete()
        response = self._get()
        self.assertRedirectsNoFollow(response, '/test')
        self.assertEqual(NokiaUser.objects.count(), 0)

    def test_next(self):
        response = self._get(get_kwargs={'next': '/next'})
        self.assertRedirectsNoFollow(response, '/test')
        self.assertEqual(
            self.client.session.get('nokia_next', None), '/next')
        self.assertEqual(NokiaUser.objects.count(), 1)


class TestCompleteView(NokiaTestBase):
    url_name = 'nokia-complete'
    access_token = 'abc'
    token_expiry = int((
        datetime.datetime.utcnow() - datetime.datetime(1970, 1, 1)
    ).total_seconds()) + 10
    token_type = 'Bearer'
    refresh_token = '123'
    nokia_user_id = 1111111

    def setUp(self):
        super(TestCompleteView, self).setUp()
        self.nokia_user.delete()

    def _get(self, use_code=True, **kwargs):
        NokiaApi.get_user = mock.MagicMock(return_value=self.get_user)
        NokiaApi.get_measures = mock.MagicMock(
            return_value=self.get_measures)
        NokiaApi.subscribe = mock.MagicMock(return_value=None)
        NokiaAuth.get_credentials = mock.MagicMock(
            return_value=NokiaCredentials(
                access_token=self.access_token,
                token_expiry=self.token_expiry,
                token_type=self.token_type,
                refresh_token=self.refresh_token,
                user_id=self.nokia_user_id))
        get_kwargs = {}
        if use_code:
            get_kwargs['code']  = 'fakecode'
        get_kwargs.update(kwargs.pop('get_kwargs', {}))
        return super(TestCompleteView, self)._get(
            get_kwargs=get_kwargs, **kwargs)

    def test_get(self):
        """
        Complete view should fetch & store user's access credentials, add the
        user's profile to the session, and retrieve all past body measures.
        """
        self.assertEqual(MeasureGroup.objects.count(), 0)
        self.assertEqual(Measure.objects.count(), 0)
        response = self._get()
        self.assertRedirectsNoFollow(
            response, utils.get_setting('NOKIA_LOGIN_REDIRECT'))
        nokia_user = NokiaUser.objects.get()
        self.assertEqual(NokiaApi.get_user.call_count, 1)
        NokiaApi.get_user.assert_called_once_with()
        self.assertEqual(NokiaApi.subscribe.call_count, 2)
        NokiaApi.subscribe.assert_has_calls([
            mock.call('http://testserver/notification/%s/' % appli,
                      'django-nokia', appli=appli) for appli in [1, 4]
        ])
        self.assertEqual(nokia_user.user, self.user)
        self.assertEqual(nokia_user.access_token, self.access_token)
        self.assertEqual(nokia_user.token_expiry, self.token_expiry)
        self.assertEqual(nokia_user.token_type, self.token_type)
        self.assertEqual(nokia_user.refresh_token, self.refresh_token)
        self.assertEqual(nokia_user.nokia_user_id, self.nokia_user_id)
        NokiaApi.get_measures.assert_called_once_with()
        self.assertEqual(MeasureGroup.objects.count(), 3)
        self.assertEqual(Measure.objects.count(), 5)

    def test_unauthenticated(self):
        """User must be logged in to access Complete view."""
        self.client.logout()
        response = self._get()
        self.assertEqual(response.status_code, 302)
        self.assertEqual(NokiaUser.objects.count(), 0)

    @freeze_time("2012-01-14T12:00:01", tz_offset=0)
    def test_next(self):
        """
        Complete view should redirect to session['nokia_next'] if available.
        """
        self._set_session_vars(nokia_next='/test')
        response = self._get()
        self.assertRedirectsNoFollow(response, '/test')
        nokia_user = NokiaUser.objects.get()
        self.assertEqual(nokia_user.user, self.user)
        self.assertEqual(nokia_user.access_token, self.access_token)
        self.assertEqual(nokia_user.token_expiry, self.token_expiry)
        self.assertEqual(nokia_user.token_type, self.token_type)
        self.assertEqual(nokia_user.refresh_token, self.refresh_token)
        self.assertEqual(nokia_user.last_update, timezone.now())
        self.assertEqual(nokia_user.nokia_user_id, self.nokia_user_id)

    def test_no_code(self):
        """Complete view should redirect to error if code isn't available."""
        response = self._get(use_code=False)

        self.assertRedirectsNoFollow(response, reverse('nokia-error'))
        self.assertEqual(NokiaUser.objects.count(), 0)
        
        # import pdb; pdb.set_trace()
        response = self._get(get_kwargs={'code': ''})

        self.assertRedirectsNoFollow(response, reverse('nokia-error'))
        self.assertEqual(NokiaUser.objects.count(), 0)

    @freeze_time("2012-01-14T12:00:01", tz_offset=0)
    def test_integrated(self):
        """
        Complete view should overwrite existing credentials for this user.
        """
        self.nokia_user = self.create_nokia_user(user=self.user)
        response = self._get()
        nokia_user = NokiaUser.objects.get()
        self.assertEqual(nokia_user.user, self.user)
        self.assertEqual(nokia_user.access_token, self.access_token)
        self.assertEqual(nokia_user.token_expiry, self.token_expiry)
        self.assertEqual(nokia_user.token_type, self.token_type)
        self.assertEqual(nokia_user.refresh_token, self.refresh_token)
        self.assertEqual(nokia_user.last_update, timezone.now())
        self.assertEqual(nokia_user.nokia_user_id, self.nokia_user_id)
        self.assertRedirectsNoFollow(
            response, utils.get_setting('NOKIA_LOGIN_REDIRECT'))


class TestErrorView(NokiaTestBase):
    url_name = 'nokia-error'

    def test_get(self):
        """Should be able to retrieve Error page."""
        response = self._get()
        self.assertEqual(response.status_code, 200)

    def test_unauthenticated(self):
        """User must be logged in to access Error view."""
        self.client.logout()
        response = self._get()
        self.assertEqual(response.status_code, 302)

    def test_unintegrated(self):
        """No Nokia credentials required to access Error view."""
        self.nokia_user.delete()
        response = self._get()
        self.assertEqual(response.status_code, 200)


class TestLogoutView(NokiaTestBase):
    url_name = 'nokia-logout'

    def setUp(self):
        super(TestLogoutView, self).setUp()
        NokiaApi.list_subscriptions = mock.MagicMock(return_value=[{
            'comment': 'django-nokia',
            'expires': 2147483647,
            'appli': 1,
            'callbackurl': 'http://testserver/notification/1/',
        }, {
            'comment': 'django-nokia',
            'expires': 2147483647,
            'appli': 4,
            'callbackurl': 'http://testserver/notification/4/',
        }])
        NokiaApi.unsubscribe = mock.MagicMock(return_value=None)

    def test_get(self):
        """Logout view should remove associated NokiaUser and redirect."""
        response = self._get()
        NokiaApi.list_subscriptions.assert_called_once_with()
        self.assertEqual(NokiaApi.unsubscribe.call_count, 2)
        NokiaApi.unsubscribe.assert_has_calls([
            mock.call('http://testserver/notification/%s/' % appli,
                      appli=appli) for appli in [1, 4]
        ])
        self.assertRedirectsNoFollow(response,
                                     utils.get_setting('NOKIA_LOGIN_REDIRECT'))
        self.assertEqual(NokiaUser.objects.count(), 0)

    def test_unauthenticated(self):
        """User must be logged in to access Logout view."""
        self.client.logout()
        response = self._get()
        self.assertEqual(response.status_code, 302)
        self.assertEqual(NokiaUser.objects.count(), 1)

    def test_unintegrated(self):
        """No Nokia credentials required to access Logout view."""
        self.nokia_user.delete()
        response = self._get()
        self.assertRedirectsNoFollow(response,
                                     utils.get_setting('NOKIA_LOGIN_REDIRECT'))
        self.assertEqual(NokiaUser.objects.count(), 0)

    def test_next(self):
        """Logout view should redirect to GET['next'] if available."""
        response = self._get(get_kwargs={'next': '/test'})
        self.assertRedirectsNoFollow(response, '/test')
        self.assertEqual(NokiaUser.objects.count(), 0)



import arrow
import datetime

from django.conf import settings
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from math import pow


UserModel = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')


@python_2_unicode_compatible
class NokiaUser(models.Model):
    """ A user's Nokia credentials, allowing API access """
    user = models.OneToOneField(UserModel, help_text='The user')
    nokia_user_id = models.IntegerField(help_text='The nokia user ID')
    access_token = models.TextField(help_text='OAuth2 access token')
    token_expiry = models.IntegerField(help_text='Token expiration timestamp')
    token_type = models.CharField(
        max_length=32, help_text='Type of OAuth2 token')
    refresh_token = models.TextField(help_text='OAuth2 refresh token')
    last_update = models.DateTimeField(
        null=True,
        blank=True,
        help_text="The datetime the user's nokia data was last updated")

    def __str__(self):
        if hasattr(self.user, 'get_username'):
            return self.user.get_username()
        else:  # Django 1.4
            return self.user.username

    def get_user_data(self):
        return {
            'access_token': self.access_token,
            'token_expiry': self.token_expiry,
            'token_type': self.token_type,
            'refresh_token': self.refresh_token,
            'user_id': self.nokia_user_id,
            'refresh_cb': self.refresh_cb,
        }

    def refresh_cb(self, token):
        self.access_token = token['access_token']
        self.token_expiry = int((
            datetime.datetime.utcnow() - datetime.datetime(1970, 1, 1)
        ).total_seconds()) + int(token['expires_in'])
        self.token_type = token['token_type']
        self.refresh_token = token['refresh_token']
        self.save()


@python_2_unicode_compatible
class MeasureGroup(models.Model):
    """
    A group of measurements, i.e. Diastolic/Systolic blood pressure:
    http://oauth.nokia.com/api/doc#api-Measure-get_measure
    """
    non_ambiguous_device = 0
    ambiguous_device = 1
    manual_entry = 2
    manual_creation_entry = 4
    ATTRIB_TYPES = (
        (non_ambiguous_device, 'Captured by a device, not ambiguous'),
        (ambiguous_device, 'Captured by a device, may belong to other user'),
        (manual_entry, 'Manually entered by user'),
        (manual_creation_entry, 'Manually entered, may not be accurate')
    )
    real = 1
    objective = 2
    CATEGORY_TYPES = (
        (real, 'Real measurements'),
        (objective, 'User objectives')
    )

    user = models.ForeignKey(UserModel, help_text="The group's user")
    grpid = models.IntegerField(help_text='The group ID, assigned by nokia')
    attrib = models.IntegerField(
        choices=ATTRIB_TYPES,
        help_text="The group's attribution, one of: {}".format(
            ', '.join(['{} ({})'.format(ci, cs) for ci, cs in ATTRIB_TYPES])
        ))
    date = models.DateTimeField(help_text='The datetime of the measurement(s)')
    updatetime = models.DateTimeField(
        help_text='The last updated datetime of the measurement(s)')
    category = models.IntegerField(
        choices=CATEGORY_TYPES,
        help_text="The group's category, one of: {}".format(
            ', '.join(['{} ({})'.format(ci, cs) for ci, cs in CATEGORY_TYPES])
        ))

    class Meta:
        unique_together = ('user', 'grpid',)

    def __str__(self):
        return '%s: %s' % (self.date.date().isoformat() if self.date else None,
                           self.get_category_display())

    @classmethod
    def create_from_measures(cls, user, measures):
        for nokia_measure in measures:
            if MeasureGroup.objects.filter(grpid=nokia_measure.grpid,
                                           user=user).exists():
                continue
            measure_grp = MeasureGroup.objects.create(
                user=user, grpid=nokia_measure.grpid,
                attrib=nokia_measure.attrib,
                category=nokia_measure.category,
                date=nokia_measure.date.datetime,
                updatetime=measures.updatetime.datetime)
            for measure in nokia_measure.measures:
                Measure.objects.create(
                    group=measure_grp, value=measure['value'],
                    measure_type=measure['type'], unit=measure['unit'])


@python_2_unicode_compatible
class Measure(models.Model):
    """
    A body measurement:
    http://oauth.nokia.com/api/doc#api-Measure-get_measure
    """
    weight = 1
    height = 4
    fat_free_mass = 5
    fat_ratio = 6
    fat_mass_weight = 8
    diastolic_bp = 9
    systolic_bp = 10
    heart_pulse = 11
    sp02 = 54
    MEASURE_TYPES = (
        (weight, 'Weight (kg)'),
        (height, 'Height (meter)'),
        (fat_free_mass, 'Fat Free Mass (kg)'),
        (fat_ratio, 'Fat Ratio (%)'),
        (fat_mass_weight, 'Fat Mass Weight (kg)'),
        (diastolic_bp, 'Diastolic Blood Pressure (mmHg)'),
        (systolic_bp, 'Systolic Blood Pressure (mmHg)'),
        (heart_pulse, 'Heart Pulse (bpm)'),
        (sp02, 'SP02(%)'),
    )

    group = models.ForeignKey(
        MeasureGroup,
        related_name='measures',
        help_text="The measurement's group")
    value = models.IntegerField(
        help_text=(
            'Value for the measure in S.I units (kilogram, meters, etc.). '
            'Value should be multiplied by 10 to the power of "unit" to get '
            'the real value.'
        ))
    measure_type = models.IntegerField(
        choices=MEASURE_TYPES,
        help_text="The measurement's type, one of: {}".format(
            ', '.join(['{} ({})'.format(ci, cs) for ci, cs in MEASURE_TYPES])
        ))
    unit = models.IntegerField(
        help_text=(
            'Power of ten the "value" attribute should be multiplied to to '
            'get the real value. Eg : value = 20 and unit=-1 means the value '
            'really is 2.0'
        ))

    def get_value(self):
        return float(self.value) * pow(10, self.unit)

    def __str__(self):
        return '%s: %s' % (self.get_measure_type_display(), self.get_value())



from mock import patch, Mock
import datetime
import random
try:
    from urllib.parse import urlencode
    from string import ascii_letters
except:
    # Python 2.x
    from urllib import urlencode
    from string import letters as ascii_letters

import django
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.test import TestCase

from nokia import NokiaApi, NokiaCredentials, NokiaMeasures

from nokiaapp.models import NokiaUser


class NokiaTestBase(TestCase):
    TEST_SERVER = 'http://testserver'

    def setUp(self):
        self.username = self.random_string(25)
        self.password = self.random_string(25)
        self.user = self.create_user(username=self.username,
                                     password=self.password)
        self.nokia_user = self.create_nokia_user(user=self.user)
        self.get_user = {
            'id': 1111111, 'birthdate': 364305600, 'lastname': 'Baggins',
            'ispublic': 255, 'firstname': 'Frodo', 'fatmethod': 131}
        self.get_measures = NokiaMeasures({
            "updatetime": 1249409679,
            "measuregrps": [{
                "grpid": 2909,
                "attrib": 0,
                "date": 1222930968,
                "category": 1,
                "measures": [{
                    "value": 79300,
                    "type": 1,
                    "unit": -3
                }]
            }, {
                "grpid": 2910,
                "attrib": 1,
                "date": 1222930968,
                "category": 1,
                "measures": [{
                    "value": 652,
                    "type": 5,
                    "unit": -1
                }, {
                    "value": 178,
                    "type": 6,
                    "unit": -1
                }, {
                    "value": 14125,
                    "type": 8,
                    "unit": -3
                }]
            },
            {
                "grpid": 2908,
                "attrib": 0,
                "date": 1222930968,
                "category": 1,
                "measures": [{
                    "value": 173,
                    "type": 4,
                    "unit": -2
                }]
            }]
        })

        self.client.login(username=self.username, password=self.password)

    def random_string(self, length=255, extra_chars=''):
        chars = ascii_letters + extra_chars
        return ''.join([random.choice(chars) for i in range(length)])

    def create_user(self, username=None, email=None, password=None, **kwargs):
        username = username or self.random_string(25)
        email = email or '{0}@{1}.com'.format(self.random_string(25),
                                              self.random_string(10))
        password = password or self.random_string(25)
        user = User.objects.create_user(username, email, password)
        User.objects.filter(pk=user.pk).update(**kwargs)
        user = User.objects.get(pk=user.pk)
        return user

    def create_nokia_user(self, **kwargs):
        defaults = {
            'user': kwargs.pop('user', self.create_user()),
            'nokia_user_id': random.randint(111111, 999999),
            'access_token': self.random_string(25),
            'token_expiry': int((
                datetime.datetime.utcnow() - datetime.datetime(1970, 1, 1)
            ).total_seconds()) + 10,  # Expires in 10 seconds
            'token_type': 'Bearer',
            'refresh_token': self.random_string(25),
        }
        defaults.update(kwargs)
        return NokiaUser.objects.create(**defaults)

    def assertRedirectsNoFollow(self, response, url, status_code=302):
        """
        Workaround to test whether a response redirects to another URL without
        loading the page at that URL.
        """
        self.assertEqual(response.status_code, status_code)
        full_url = (self.TEST_SERVER if django.VERSION < (1, 9,) else '') + url
        self.assertEqual(response._headers['location'][1], full_url)

    def _get(self, url_name=None, url_kwargs=None, get_kwargs=None, **kwargs):
        """Convenience wrapper for test client GET request."""
        url_name = url_name or self.url_name
        url = reverse(url_name, kwargs=url_kwargs)  # Base URL.

        # Add GET parameters.
        if get_kwargs:
            url += '?' + urlencode(get_kwargs)

        return self.client.get(url, **kwargs)

    def _set_session_vars(self, **kwargs):
        session = self.client.session
        for key, value in kwargs.items():
            session[key] = value
        try:
            session.save()  # Only available on authenticated sessions.
        except AttributeError:
            pass

    def _error_response(self):
        error_response = Mock(['content'])
        error_response.content = '{"errors": []}'.encode('utf8')
        return error_response

    @patch('nokiaapp.utils.get_nokia_data')
    def _mock_utility(self, utility=None, error=None, response=None, **kwargs):
        if error:
            utility.side_effect = error(self._error_response())
        elif response:
            utility.return_value = response
        return self._get(**kwargs)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2018-09-11 15:33
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('nokiaapp', '0005_migrate_oauth_tokens'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='nokiauser',
            name='access_token_secret',
        ),
    ]



import arrow
import datetime

from django.db import IntegrityError
from nokia import NokiaCredentials, NokiaMeasures
from nokiaapp.models import NokiaUser, Measure, MeasureGroup

from .base import NokiaTestBase


class TestNokiaModels(NokiaTestBase):
    def test_nokia_user(self):
        """
        NokiaUser was already created in base, now test the
        properties
        """
        self.assertEqual(self.nokia_user.user, self.user)
        self.assertEqual(self.nokia_user.__str__(), self.username)
        self.assertEqual(self.nokia_user.last_update, None)
        data = self.nokia_user.get_user_data()
        self.assertEqual(type(data), dict)
        self.assertEqual(data['access_token'], self.nokia_user.access_token)
        self.assertEqual(data['token_expiry'], self.nokia_user.token_expiry)
        self.assertEqual(data['token_type'], self.nokia_user.token_type)
        self.assertEqual(data['refresh_token'], self.nokia_user.refresh_token)
        self.assertEqual(data['user_id'], self.nokia_user.nokia_user_id)
        self.assertEqual(data['refresh_cb'], self.nokia_user.refresh_cb)

        token = {
            'access_token': 'newat',
            'token_type': 'newtype',
            'expires_in': 100,
            'refresh_token': 'newrt',
        }
        now_stamp = int((
            datetime.datetime.utcnow() - datetime.datetime(1970, 1, 1)
        ).total_seconds())

        self.nokia_user.refresh_cb(token)

        self.assertEqual(self.nokia_user.access_token, token['access_token'])
        self.assertEqual(self.nokia_user.token_type, token['token_type'])
        # Check 100 or 101 in case a second ticked over during testing
        self.assertTrue(
            self.nokia_user.token_expiry == (now_stamp + 100) or
            self.nokia_user.token_expiry == (now_stamp + 101)
        )
        self.assertEqual(self.nokia_user.refresh_token, token['refresh_token'])

    def test_measure_group(self):
        """ Create a MeasureGroup model, check attributes and methods """
        measures = NokiaMeasures({
            "updatetime": 1249409679,
            "measuregrps": [{
                 "grpid": 2909,
                 "attrib": 0,
                 "date": 1222930968,
                 "category": 1,
                 "measures": []
            }]
        })
        self.assertEqual(len(measures), 1)
        measure = measures[0]
        measure_grp = MeasureGroup.objects.create(
            user=self.user, grpid=measure.grpid, attrib=measure.attrib,
            date=measure.date.datetime, category=measure.category,
            updatetime=measures.updatetime.datetime)
        self.assertEqual(measure_grp.__str__(),
                         '2008-10-02: Real measurements')
        self.assertEqual(measure_grp.grpid, 2909)
        self.assertEqual(measure_grp.attrib, 0)
        self.assertEqual(arrow.get(measure_grp.date).timestamp, 1222930968)
        self.assertEqual(arrow.get(measure_grp.updatetime).timestamp,
                         1249409679)
        self.assertEqual(measure_grp.category, 1)
        self.assertEqual(measure_grp.get_attrib_display(),
                         'Captured by a device, not ambiguous')
        self.assertEqual(measure_grp.non_ambiguous_device, measure_grp.attrib)
        self.assertEqual(measure_grp.get_category_display(),
                         'Real measurements')
        self.assertEqual(measure_grp.real, measure_grp.category)

        # Test the create_from_measures function
        MeasureGroup.objects.all().delete()
        measures = self.get_measures
        self.assertEqual(MeasureGroup.objects.count(), 0)
        self.assertEqual(Measure.objects.count(), 0)
        MeasureGroup.create_from_measures(self.user, measures)
        self.assertEqual(MeasureGroup.objects.count(), 3)
        self.assertEqual(Measure.objects.count(), 5)
        self.assertEqual(
            MeasureGroup.objects.get(grpid=2908).measures.count(), 1)
        self.assertEqual(
            MeasureGroup.objects.get(grpid=2909).measures.count(), 1)
        self.assertEqual(
            MeasureGroup.objects.get(grpid=2910).measures.count(), 3)
        # create_from_measures should silently ignore duplicates
        try:
            MeasureGroup.create_from_measures(self.user, measures)
            self.assertEqual(MeasureGroup.objects.count(), 3)
            self.assertEqual(Measure.objects.count(), 5)
        except IntegrityError:
            assert False, 'Not ignoring duplicates'
        # Can't create MeasureGroup with the same user and grpid
        self.assertRaises(IntegrityError, MeasureGroup.objects.create,
            user=self.user, grpid=2908, attrib=1, category=2,
            date=measures[0].date.datetime,
            updatetime=measures.updatetime.datetime)

    def test_measure(self):
        """ Create a Measure model, check attributes and methods """
        nokia_measures = NokiaMeasures({
            "updatetime": 1249409679,
            "measuregrps": [{
                 "grpid": 2909,
                 "attrib": 0,
                 "date": 1222930968,
                 "category": 1,
                 "measures": []
            }]
        })
        nokia_measure = nokia_measures[0]
        measure_grp = MeasureGroup.objects.create(
            user=self.user, grpid=nokia_measure.grpid,
            attrib=nokia_measure.attrib, category=nokia_measure.category,
            date=nokia_measure.date.datetime,
            updatetime=nokia_measures.updatetime.datetime)
        measure = Measure.objects.create(
            group=measure_grp, value=79300, measure_type=1, unit=-3)
        self.assertEqual(measure.__str__(), 'Weight (kg): 79.3')
        self.assertEqual(measure.value, 79300)
        self.assertEqual(measure.measure_type, 1)
        self.assertEqual(measure.unit, -3)
        self.assertEqual(measure.get_value(), 79.3)
        self.assertEqual(measure.get_measure_type_display(), 'Weight (kg)')
        self.assertEqual(measure.weight, 1)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from nokia import NokiaApi, NokiaAuth, NokiaCredentials

from . import defaults
from .models import NokiaUser


def create_nokia(client_id=None, consumer_secret=None, **kwargs):
    """ Shortcut to create a NokiaApi instance. """

    refresh_cb = kwargs.pop('refresh_cb', None)
    return NokiaApi(get_creds(
        client_id=client_id,
        consumer_secret=consumer_secret,
        **kwargs
    ), refresh_cb=refresh_cb)


def create_nokia_auth(callback_uri):
    creds = get_creds()

    return NokiaAuth(creds.client_id, creds.consumer_secret, callback_uri)


def get_creds(client_id=None, consumer_secret=None, **kwargs):
    """
    If client_id or consumer_secret are not provided, then the values specified
    in settings are used.
    """
    if client_id is None:
        client_id = get_setting('NOKIA_CLIENT_ID')
    if consumer_secret is None:
        consumer_secret = get_setting('NOKIA_CONSUMER_SECRET')

    if not all([client_id, consumer_secret]):
        raise ImproperlyConfigured(
            "Neither lient id nor consumer secret can be null, they must be "
            "explicitly specified or set in your Django settings")

    return NokiaCredentials(
        client_id=client_id, consumer_secret=consumer_secret, **kwargs)


def is_integrated(user):
    """Returns ``True`` if we have Oauth info for the user.

    This does not require that the token and secret are valid.

    :param user: A Django User.
    """
    if user.is_authenticated() and user.is_active:
        return NokiaUser.objects.filter(user=user).exists()
    return False


def get_nokia_data(nokia_user, **kwargs):
    """
    Retrieves nokia data for the date range
    """
    api = create_nokia(**nokia_user.get_user_data())
    return api.get_measures(**kwargs)


def get_setting(name, use_defaults=True):
    """Retrieves the specified setting from the settings file.

    If the setting is not found and use_defaults is True, then the default
    value specified in defaults.py is used. Otherwise, we raise an
    ImproperlyConfigured exception for the setting.
    """
    if hasattr(settings, name):
        return getattr(settings, name)
    if use_defaults:
        if hasattr(defaults, name):
            return getattr(defaults, name)
    msg = "{0} must be specified in your settings".format(name)
    raise ImproperlyConfigured(msg)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2017-06-29 21:57
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


def copy_to_nokia(apps, schema_editor):
    WithingsUser = apps.get_model('nokiaapp', 'WithingsUser')
    NokiaUser = apps.get_model('nokiaapp', 'NokiaUser')
    for user in WithingsUser.objects.all():
        NokiaUser.objects.create(
            id=user.id,
            nokia_user_id=user.withings_user_id,
            access_token=user.access_token,
            access_token_secret=user.access_token_secret,
            last_update=user.last_update,
            user=user.user
        )


def copy_to_withings(apps, schema_editor):
    WithingsUser = apps.get_model('nokiaapp', 'WithingsUser')
    NokiaUser = apps.get_model('nokiaapp', 'NokiaUser')
    for user in NokiaUser.objects.all():
        WithingsUser.objects.create(
            id=user.id,
            withings_user_id=user.nokia_user_id,
            access_token=user.access_token,
            access_token_secret=user.access_token_secret,
            last_update=user.last_update,
            user=user.user
        )


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('nokiaapp', '0002_help_text'),
    ]

    operations = [
        migrations.CreateModel(
            name='NokiaUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('nokia_user_id', models.IntegerField(help_text=b'The nokia user ID')),
                ('access_token', models.TextField(help_text=b'OAuth access token')),
                ('access_token_secret', models.TextField(help_text=b'OAuth access token secret')),
                ('last_update', models.DateTimeField(blank=True, help_text=b"The datetime the user's nokia data was last updated", null=True)),
                ('user', models.OneToOneField(help_text=b'The user', on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.RunPython(copy_to_nokia, copy_to_withings),
        migrations.RemoveField(
            model_name='withingsuser',
            name='user',
        ),
        migrations.AlterField(
            model_name='measure',
            name='group',
            field=models.ForeignKey(help_text=b"The measurement's group", on_delete=django.db.models.deletion.CASCADE, related_name='measures', to='nokiaapp.MeasureGroup'),
        ),
        migrations.AlterField(
            model_name='measuregroup',
            name='grpid',
            field=models.IntegerField(help_text=b'The group ID, assigned by nokia'),
        ),
        migrations.AlterModelTable(
            name='measure',
            table=None,
        ),
        migrations.AlterModelTable(
            name='measuregroup',
            table=None,
        ),
        migrations.DeleteModel(
            name='WithingsUser',
        ),
    ]



import arrow

from django.core.urlresolvers import reverse
from freezegun import freeze_time

from nokia import NokiaMeasures

from nokiaapp.models import NokiaUser, MeasureGroup, Measure

from .base import NokiaTestBase

try:
    from unittest import mock
except ImportError:  # Python 2.x fallback
    import mock


class TestRetrievalTask(NokiaTestBase):
    def setUp(self):
        super(TestRetrievalTask, self).setUp()
        self.startdate = 1222930967
        self.enddate = 1222930969
        self.nokia_measures = {
            "updatetime": 1249409679,
            "measuregrps": [{
                "grpid": 2909,
                "attrib": 0,
                "date": 1222930968,
                "category": 1,
                "measures": [{
                    "value": 79300,
                    "type": 1,
                    "unit": -3
                }]
            }, {
                "grpid": 2910,
                "attrib": 1,
                "date": 1222930968,
                "category": 1,
                "measures": [{
                    "value": 652,
                    "type": 5,
                    "unit": -1
                }, {
                    "value": 178,
                    "type": 6,
                    "unit": -1
                }, {
                    "value": 14125,
                    "type": 8,
                    "unit": -3
                }]
            }, {
                "grpid": 2908,
                "attrib": 0,
                "date": 1222930968,
                "category": 1,
                "measures": [{
                    "value": 173,
                    "type": 4,
                    "unit": -2
                }]
            }]
        }

    def _receive_nokia_notification(self, status_code=204):
        post_params = {
            'userid': self.nokia_user.nokia_user_id,
            'startdate': self.startdate,
            'enddate': self.enddate
        }
        res = self.client.post(
            reverse('nokia-notification', kwargs={'appli': 1}),
            data=post_params)
        self.assertEqual(res.status_code, status_code)

    def test_notification_get(self):
        res = self.client.get(
            reverse('nokia-notification', kwargs={'appli': 1}))

        self.assertEqual(res.status_code, 404)

    def test_notification_head(self):
        res = self.client.head(
            reverse('nokia-notification', kwargs={'appli': 1}))

        self.assertEqual(res.status_code, 200)

    @freeze_time("2012-01-14T12:00:01", tz_offset=-6)
    @mock.patch('nokiaapp.utils.get_nokia_data')
    def test_notification(self, get_nokia_data):
        # Check that data is created when a notification is received from Nokia
        get_nokia_data.return_value = NokiaMeasures(self.nokia_measures)
        self.assertEqual(self.nokia_user.last_update, None)
        self.assertEqual(MeasureGroup.objects.count(), 0)
        self.assertEqual(Measure.objects.count(), 0)
        self._receive_nokia_notification()
        self.assertEqual(get_nokia_data.call_count, 1)
        self.nokia_user = NokiaUser.objects.get(id=self.nokia_user.id)
        self.assertEqual(self.nokia_user.last_update.isoformat(),
                         "2012-01-14T12:00:01+00:00")
        self.assertEqual(MeasureGroup.objects.count(), 3)
        self.assertEqual(Measure.objects.count(), 5)
        startdate = arrow.get(self.startdate).datetime
        enddate = arrow.get(self.enddate).datetime
        measure_grps = MeasureGroup.objects.filter(
            user=self.user, date__gte=startdate, date__lte=enddate)
        self.assertEqual(measure_grps.count(), 3)

    @freeze_time("2012-01-14T12:00:01", tz_offset=-6)
    @mock.patch('nokiaapp.utils.get_nokia_data')
    def test_notification_multi_user_with_error(self, get_nokia_data):
        # Check that the notification handler gracefully handles errors
        get_nokia_data.side_effect = [
            Exception("Error code 283"),
            NokiaMeasures(self.nokia_measures),
        ]
        user2 = self.create_user(username=self.username + '2',
                                 password=self.password)
        nokia_user2 = self.create_nokia_user(user=user2)
        nokia_user2.nokia_user_id = self.nokia_user.nokia_user_id
        nokia_user2.save()

        self.assertEqual(self.nokia_user.last_update, None)
        self.assertEqual(MeasureGroup.objects.count(), 0)
        self.assertEqual(Measure.objects.count(), 0)

        self._receive_nokia_notification()

        self.assertEqual(get_nokia_data.call_count, 2)
        # Only one of the users should have received data
        self.assertEqual(MeasureGroup.objects.count(), 3)
        self.assertEqual(Measure.objects.count(), 5)
        self.assertEqual(MeasureGroup.objects.count(), 3)

    def test_notification_error(self):
        res = self.client.post(
            reverse('nokia-notification', kwargs={'appli': 4}))
        self.assertEqual(res.status_code, 404)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('nokiaapp', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='measure',
            name='group',
            field=models.ForeignKey(help_text=b"The measurement's group", to='MeasureGroup'),
        ),
        migrations.AlterField(
            model_name='measure',
            name='measure_type',
            field=models.IntegerField(choices=[(1, b'Weight (kg)'), (4, b'Height (meter)'), (5, b'Fat Free Mass (kg)'), (6, b'Fat Ratio (%)'), (8, b'Fat Mass Weight (kg)'), (9, b'Diastolic Blood Pressure (mmHg)'), (10, b'Systolic Blood Pressure (mmHg)'), (11, b'Heart Pulse (bpm)'), (54, b'SP02(%)')], help_text=b"The measurement's type, one of: 1 (Weight (kg)), 4 (Height (meter)), 5 (Fat Free Mass (kg)), 6 (Fat Ratio (%)), 8 (Fat Mass Weight (kg)), 9 (Diastolic Blood Pressure (mmHg)), 10 (Systolic Blood Pressure (mmHg)), 11 (Heart Pulse (bpm)), 54 (SP02(%))"),
        ),
        migrations.AlterField(
            model_name='measure',
            name='unit',
            field=models.IntegerField(help_text=b'Power of ten the "value" attribute should be multiplied to to get the real value. Eg : value = 20 and unit=-1 means the value really is 2.0'),
        ),
        migrations.AlterField(
            model_name='measure',
            name='value',
            field=models.IntegerField(help_text=b'Value for the measure in S.I units (kilogram, meters, etc.). Value should be multiplied by 10 to the power of "unit" to get the real value.'),
        ),
        migrations.AlterField(
            model_name='measuregroup',
            name='attrib',
            field=models.IntegerField(choices=[(0, b'Captured by a device, not ambiguous'), (1, b'Captured by a device, may belong to other user'), (2, b'Manually entered by user'), (4, b'Manually entered, may not be accurate')], help_text=b"The group's attribution, one of: 0 (Captured by a device, not ambiguous), 1 (Captured by a device, may belong to other user), 2 (Manually entered by user), 4 (Manually entered, may not be accurate)"),
        ),
        migrations.AlterField(
            model_name='measuregroup',
            name='category',
            field=models.IntegerField(choices=[(1, b'Real measurements'), (2, b'User objectives')], help_text=b"The group's category, one of: 1 (Real measurements), 2 (User objectives)"),
        ),
        migrations.AlterField(
            model_name='measuregroup',
            name='date',
            field=models.DateTimeField(help_text=b'The datetime of the measurement(s)'),
        ),
        migrations.AlterField(
            model_name='measuregroup',
            name='grpid',
            field=models.IntegerField(help_text=b'The group ID, assigned by withings'),
        ),
        migrations.AlterField(
            model_name='measuregroup',
            name='updatetime',
            field=models.DateTimeField(help_text=b'The last updated datetime of the measurement(s)'),
        ),
        migrations.AlterField(
            model_name='measuregroup',
            name='user',
            field=models.ForeignKey(help_text=b"The group's user", to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='withingsuser',
            name='access_token',
            field=models.TextField(help_text=b'OAuth access token'),
        ),
        migrations.AlterField(
            model_name='withingsuser',
            name='access_token_secret',
            field=models.TextField(help_text=b'OAuth access token secret'),
        ),
        migrations.AlterField(
            model_name='withingsuser',
            name='last_update',
            field=models.DateTimeField(blank=True, null=True, help_text=b"The datetime the user's withings data was last updated"),
        ),
        migrations.AlterField(
            model_name='withingsuser',
            name='user',
            field=models.OneToOneField(help_text=b'The user', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='withingsuser',
            name='withings_user_id',
            field=models.IntegerField(help_text=b'The withings user ID'),
        ),
    ]



from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase
from nokia import NokiaApi

from nokiaapp.utils import create_nokia, get_setting


class TestNokiaUtilities(TestCase):
    def test_create_nokia(self):
        """
        Check that the create_nokia utility creates a WithingnsApi object
        and an error is raised when the consumer key or secret aren't set.
        """
        with self.settings(NOKIA_CLIENT_ID=None, NOKIA_CONSUMER_SECRET=None):
            self.assertRaises(ImproperlyConfigured, create_nokia)
        with self.settings(NOKIA_CLIENT_ID='a', NOKIA_CONSUMER_SECRET=None):
            self.assertRaises(ImproperlyConfigured, create_nokia)
        with self.settings(NOKIA_CLIENT_ID=None, NOKIA_CONSUMER_SECRET='a'):
            self.assertRaises(ImproperlyConfigured, create_nokia)
        api = create_nokia(token_expiry=1534796425)
        self.assertEqual(type(api), NokiaApi)
        self.assertEqual(api.credentials.client_id,
                         get_setting('NOKIA_CLIENT_ID'))
        self.assertEqual(api.credentials.consumer_secret,
                         get_setting('NOKIA_CONSUMER_SECRET'))
        self.assertEqual(api.credentials.token_expiry, 1534796425)
        self.assertTrue(int(api.token['expires_in']) < 0)

    def test_get_setting_error(self):
        """
        Check that an error is raised when trying to get a nonexistent setting.
        """
        self.assertRaises(ImproperlyConfigured, get_setting, 'DOES_NOT_EXIST')



from setuptools import setup, find_packages

required = [line for line in open('requirements/base.txt').read().split("\n")]

setup(
    name="django-nokia",
    version=__import__("nokiaapp").__version__,
    author="orcas",
    author_email="bpitcher@orcasinc.com",
    packages=find_packages(),
    install_requires=["setuptools"] + required,
    include_package_data=True,
    url="https://github.com/orcasgit/django-nokia/",
    license="License :: OSI Approved :: Apache Software License",
    description="Django integration for python-nokia",
    long_description=open("README.md").read(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        'License :: OSI Approved :: Apache Software License',
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation",
        "Programming Language :: Python :: Implementation :: PyPy"
    ],
    test_suite="runtests.runtests"
)



import logging

from django.contrib.auth.decorators import login_required
from django.contrib.auth.signals import user_logged_in
from django.core.urlresolvers import reverse
from django.dispatch import receiver
from django.http import HttpResponse, Http404
from django.shortcuts import redirect, render
from django.utils import timezone
from django.views.decorators.csrf import csrf_exempt

from . import utils
from .models import NokiaUser, MeasureGroup

try:
    from django.urls import NoReverseMatch
except ImportError:
    # Fallback for older Djangos
    from django.core.urlresolvers import NoReverseMatch

logger = logging.getLogger(__name__)


@login_required
def login(request):
    """
    Begins the OAuth authentication process by obtaining a Request Token from
    Nokia and redirecting the user to the Nokia site for authorization.

    When the user has finished at the Nokia site, they will be redirected
    to the :py:func:`nokiaapp.views.complete` view.

    If 'next' is provided in the GET data, it is saved in the session so the
    :py:func:`nokiaapp.views.complete` view can redirect the user to that
    URL upon successful authentication.

    URL name:
        `nokia-login`
    """
    next_url = request.GET.get('next', None)
    if next_url:
        request.session['nokia_next'] = next_url
    else:
        request.session.pop('nokia_next', None)

    callback_uri = request.build_absolute_uri(reverse('nokia-complete'))
    auth = utils.create_nokia_auth(callback_uri)
    auth_url = auth.get_authorize_url()
    return redirect(auth_url)


@login_required
def complete(request):
    """
    After the user authorizes us, Nokia sends a callback to this URL to
    complete authentication.

    If there was an error, the user is redirected again to the `error` view.

    If the authorization was successful, the credentials are stored for us to
    use later, and the user is redirected. If 'next_url' is in the request
    session, the user is redirected to that URL. Otherwise, they are
    redirected to the URL specified by the setting
    :ref:`NOKIA_LOGIN_REDIRECT`.

    If :ref:`NOKIA_SUBSCRIBE` is set to True, add a subscription to user
    data at this time.

    URL name:
        `nokia-complete`
    """
    callback_uri = request.build_absolute_uri(reverse('nokia-complete'))
    auth = utils.create_nokia_auth(callback_uri)
    try:
        code = request.GET.get('code')
    except KeyError:
        return redirect(reverse('nokia-error'))
    if not code:
        return redirect(reverse('nokia-error'))
    try:
        creds = auth.get_credentials(code)
    except:
        return redirect(reverse('nokia-error'))

    user_updates = {
        'access_token': creds.access_token,
        'token_expiry': creds.token_expiry,
        'token_type': creds.token_type,
        'refresh_token': creds.refresh_token,
        'nokia_user_id': creds.user_id,
        'last_update': timezone.now(),
    }
    nokia_user = NokiaUser.objects.filter(user=request.user)
    if nokia_user.exists():
        nokia_user.update(**user_updates)
        nokia_user = nokia_user[0]
    else:
        user_updates['user'] = request.user
        nokia_user = NokiaUser.objects.create(**user_updates)
    # Add the Nokia user info to the session
    api = utils.create_nokia(**nokia_user.get_user_data())
    request.session['nokia_profile'] = api.get_user()
    MeasureGroup.create_from_measures(request.user, api.get_measures())
    if utils.get_setting('NOKIA_SUBSCRIBE'):
        for appli in [1, 4]:
            notification_url = request.build_absolute_uri(
                reverse('nokia-notification', kwargs={'appli': appli}))
            api.subscribe(notification_url, 'django-nokia', appli=appli)

    next_url = request.session.pop('nokia_next', None) or utils.get_setting(
        'NOKIA_LOGIN_REDIRECT')
    return redirect(next_url)


@receiver(user_logged_in)
def create_nokia_session(sender, request, user, **kwargs):
    """ If the user is a Nokia user, update the profile in the session. """

    if (user.is_authenticated() and utils.is_integrated(user) and
            user.is_active):
        nokia_user = NokiaUser.objects.filter(user=user)
        if nokia_user.exists():
            api = utils.create_nokia(**nokia_user[0].get_user_data())
            try:
                request.session['nokia_profile'] = api.get_user()
            except:
                pass


@login_required
def error(request):
    """
    The user is redirected to this view if we encounter an error acquiring
    their Nokia credentials. It renders the template defined in the setting
    :ref:`NOKIA_ERROR_TEMPLATE`. The default template, located at
    *nokia/error.html*, simply informs the user of the error::

        <html>
            <head>
                <title>Nokia Authentication Error</title>
            </head>
            <body>
                <h1>Nokia Authentication Error</h1>

                <p>We encontered an error while attempting to authenticate you
                through Nokia.</p>
            </body>
        </html>

    URL name:
        `nokia-error`
    """
    return render(request, utils.get_setting('NOKIA_ERROR_TEMPLATE'), {})


@login_required
def logout(request):
    """Forget this user's Nokia credentials.

    If the request has a `next` parameter, the user is redirected to that URL.
    Otherwise, they're redirected to the URL defined in the setting
    :ref:`NOKIA_LOGOUT_REDIRECT`.

    URL name:
        `nokia-logout`
    """
    nokia_user = NokiaUser.objects.filter(user=request.user)
    urls = []
    for appli in [1, 4]:
        for app in ['nokia', 'withings']:
            try:
                urls.append(request.build_absolute_uri(reverse(
                    '{}-notification'.format(app),
                    kwargs={'appli': appli}
                )))
            except NoReverseMatch:
                # The library user does not have the legacy withings URLs
                pass
    if nokia_user.exists() and utils.get_setting('NOKIA_SUBSCRIBE'):
        try:
            api = utils.create_nokia(**nokia_user[0].get_user_data())
            subs = api.list_subscriptions()
            for sub in subs:
                if sub['callbackurl'] in urls:
                    api.unsubscribe(sub['callbackurl'], appli=sub['appli'])
        except:
            return redirect(reverse('nokia-error'))
    nokia_user.delete()
    next_url = request.GET.get('next', None) or utils.get_setting(
        'NOKIA_LOGOUT_REDIRECT')
    return redirect(next_url)


@csrf_exempt
def notification(request, appli):
    """ Receive notification from Nokia.

    More information here:
    https://developer.health.nokia.com/api/doc#api-Notification-Notification_callback

    URL name:
        `nokia-notification`
    """
    if request.method == 'HEAD':
        return HttpResponse()

    # The updates come in as a POST request with the necessary data
    uid = request.POST.get('userid')

    if uid and request.method == 'POST':
        for user in NokiaUser.objects.filter(nokia_user_id=uid):
            kwargs = {}
            if user.last_update:
                kwargs['lastupdate'] = user.last_update
            try:
                measures = utils.get_nokia_data(user, **kwargs)
            except Exception:
                logger.exception("Error getting nokia user measures")
            else:
                MeasureGroup.create_from_measures(user.user, measures)
                user.last_update = timezone.now()
                user.save()
        return HttpResponse(status=204)

    # If GET request or POST with bad data, raise a 404
    raise Http404



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-08-20 20:20
from __future__ import unicode_literals

import datetime

from django.conf import settings
from django.db import migrations

from nokia import NokiaAuth

from nokiaapp.utils import get_setting


def migrate_to_oauth2(apps, schema_editor):
    NokiaUser = apps.get_model("nokiaapp", "NokiaUser")
    auth = NokiaAuth(
        get_setting('NOKIA_CLIENT_ID'), get_setting('NOKIA_CONSUMER_SECRET'))
    for user in NokiaUser.objects.all():
        try:
            token = auth.migrate_from_oauth1(
                user.access_token,
                user.access_token_secret
            )
            user.access_token = token['access_token']
            user.token_expiry = int((
                datetime.datetime.utcnow() - datetime.datetime(1970, 1, 1)
            ).total_seconds()) + int(token['expires_in'])
            user.token_type = token['token_type']
            user.refresh_token = token['refresh_token']
            user.save()
        except:
            # Problem with existing token
            user.delete()


class Migration(migrations.Migration):

    dependencies = [
        ('nokiaapp', '0004_add_oauth2_fields'),
    ]

    operations = [
        migrations.RunPython(migrate_to_oauth2, migrations.RunPython.noop),
    ]
